package POWER::Utils::IO;

use version; $VERSION = qv('0.6.5');    # update POD & Changes & README

use 5.022000;
use warnings;
use strict;
use feature 'refaliasing';
no warnings 'experimental::refaliasing';
use Carp;

use Scalar::Util qw( openhandle weaken );
use Socket;
use Socket qw( IPPROTO_TCP SOL_SOCKET SO_KEEPALIVE );
use constant TCP_KEEPIDLE   => 4;
use constant TCP_KEEPINTVL  => 5;
use constant TCP_KEEPCNT    => 6;

use Errno qw( EAGAIN EINTR EINPROGRESS );
use Fcntl qw( F_GETFL F_SETFL O_NONBLOCK );

use Export::Attrs;
use EV;

use POWER::Utils;


my $BLOCK_SIZE      = 10240;    # Default block size for read/write

# Cache for speed.
my $EAGAIN          = EAGAIN;
my $EINTR           = EINTR;
my $EINPROGRESS     = EINPROGRESS;
my $F_GETFL         = F_GETFL;
my $F_SETFL         = F_SETFL;
my $O_NONBLOCK      = O_NONBLOCK;
my $TCP             = getprotobyname('tcp');
my $UDP             = getprotobyname('udp');
my $AF_INET         = AF_INET;
my $AF_UNIX         = AF_UNIX;
my $SOCK_STREAM     = SOCK_STREAM;
my $SOCK_DGRAM      = SOCK_DGRAM;
my $SOL_SOCKET      = SOL_SOCKET;
my $SO_REUSEADDR    = SO_REUSEADDR;
my $REUSEADDR_VAL   = pack('L!', 1);
my $INADDR_ANY      = INADDR_ANY;
my $INADDR_BROADCAST= INADDR_BROADCAST;
my $INADDR_LOOPBACK = INADDR_LOOPBACK;
my $SOMAXCONN       = SOMAXCONN;

sub tcp_client :Export(:DEFAULT) {
    croak q{usage: tcp_client HOST, PORT}   if @_ != 2;
    my ($host, $port) = @_;
    croak q{tcp_client: $_[0] undefined}    if !defined $host;
    croak q{tcp_client: $_[1] undefined}    if !defined $port;
    my $in_addr = inet_aton($host) or croak qq{can't resolve $host};
    socket(my $sock, $AF_INET, $SOCK_STREAM, $TCP)  or croak qq{socket: $!};
    Nonblock($sock);
    connect($sock, sockaddr_in($port, $in_addr))    
        or ($! != $EINPROGRESS && $! != $EAGAIN && croak qq{connect: $!});
    return $sock;
}

sub udp_client :Export(:DEFAULT) {
    croak q{usage: udp_client [HOST, PORT]}   if @_ != 0 && @_ != 2;
    socket(my $sock, $AF_INET, $SOCK_DGRAM, $UDP)  or croak qq{socket: $!};
    Nonblock($sock);
    if (@_) {
        my ($host, $port) = @_;
        croak q{udp_client: $_[0] undefined}    if !defined $host;
        croak q{udp_client: $_[1] undefined}    if !defined $port;
        my $in_addr = inet_aton($host) or croak qq{can't resolve $host};
        connect($sock, sockaddr_in($port, $in_addr)) or croak qq{connect: $!};
    }
    return $sock;
}

sub unix_client :Export(:DEFAULT) {
    croak q{usage: unix_client PATH}        if @_ != 1;
    my ($path) = @_;
    croak q{unix_client: $_[0] undefined}   if !defined $path;
    socket(my $sock, $AF_UNIX, $SOCK_STREAM, 0) or croak qq{socket: $!};
    Nonblock($sock);
    connect($sock, sockaddr_un($path))
        or ($! != $EINPROGRESS && $! != $EAGAIN && croak qq{connect: $!});
    return $sock;
}

sub tcp_server :Export(:DEFAULT) {
    croak q{usage: tcp_server HOST, PORT}   if @_ != 2;
    my ($host, $port) = @_;
    croak q{tcp_server: $_[0] undefined}    if !defined $host;
    croak q{tcp_server: $_[1] undefined}    if !defined $port;
    my $in_addr = $host eq 'ANY'        ? $INADDR_ANY
                : $host eq 'BROADCAST'  ? $INADDR_BROADCAST
                : $host eq 'LOOPBACK'   ? $INADDR_LOOPBACK
                :                         inet_aton($host)
        or croak qq{can't resolve $host};
    socket(my $sock, $AF_INET, $SOCK_STREAM, $TCP)  or croak qq{socket: $!};
    setsockopt($sock, $SOL_SOCKET, $SO_REUSEADDR, $REUSEADDR_VAL)
        or croak qq{setsockopt: $!};
    bind($sock, sockaddr_in($port, $in_addr))       or croak qq{bind: $!};
    listen($sock, $SOMAXCONN)                       or croak qq{listen: $!};
    Nonblock($sock);
    return $sock;
}

sub udp_server :Export(:DEFAULT) {
    croak q{usage: udp_server HOST, PORT}   if @_ != 2;
    my ($host, $port) = @_;
    croak q{udp_server: $_[0] undefined}    if !defined $host;
    croak q{udp_server: $_[1] undefined}    if !defined $port;
    my $in_addr = $host eq 'ANY'        ? $INADDR_ANY
                : $host eq 'BROADCAST'  ? $INADDR_BROADCAST
                : $host eq 'LOOPBACK'   ? $INADDR_LOOPBACK
                :                         inet_aton($host)
        or croak qq{can't resolve $host};
    socket(my $sock, $AF_INET, $SOCK_DGRAM, $UDP)  or croak qq{socket: $!};
    setsockopt($sock, $SOL_SOCKET, $SO_REUSEADDR, $REUSEADDR_VAL)
        or croak qq{setsockopt: $!};
    bind($sock, sockaddr_in($port, $in_addr))       or croak qq{bind: $!};
    Nonblock($sock);
    return $sock;
}

sub unix_server :Export(:DEFAULT) {
    croak q{usage: unix_server PATH}        if @_ != 1;
    my ($path) = @_;
    croak q{unix_server: $_[0] undefined}   if !defined $path;
    socket(my $sock, $AF_UNIX, $SOCK_STREAM, 0)     or croak qq{socket: $!};
    unlink($path);
    my $umask = umask 0;
    bind($sock, sockaddr_un($path))                 or croak qq{bind: $!};
    umask $umask;
    listen($sock, $SOMAXCONN)                       or croak qq{listen: $!};
    Nonblock($sock);
    return $sock;
}

sub start_server :Export {
    my ($host, $port, $cb, @params) = @_;
    my $self = {};
    $self->{sock} = tcp_server($host, $port);
    ($self->{port}) = sockaddr_in(getsockname $self->{sock});
    weaken(my $this = $self);
    $self->{w} = EV::io($self->{sock}, EV::READ, sub { if ($this) {
        if (accept my $sock, $this->{sock}) {
            Nonblock($sock);
            $cb->($sock, @params);
        }
        elsif ($! != $EAGAIN) {
            croak qq{accept: $!};
        }
    } });
    return $self;
}

sub keep_alive :Export(:DEFAULT) {
    my ($fh, $idle, $cnt, $intvl) = @_;
    $idle   ||= 60;
    $cnt    ||= 6;
    $intvl  ||= 10;
    setsockopt $fh, SOL_SOCKET,  SO_KEEPALIVE,  pack 'l', 1         or die "setsockopt: $!";
    setsockopt $fh, IPPROTO_TCP, TCP_KEEPIDLE,  pack 'l', $idle     or die "setsockopt: $!"; 
    setsockopt $fh, IPPROTO_TCP, TCP_KEEPCNT,   pack 'l', $cnt      or die "setsockopt: $!"; 
    setsockopt $fh, IPPROTO_TCP, TCP_KEEPINTVL, pack 'l', $intvl    or die "setsockopt: $!"; 
    return;
}

sub get_peer :Export {
    my ($sock) = @_;
    return (q{}, 0) if !$sock;
    my $sockaddr = getpeername $sock;
    return (q{}, 0) if !$sockaddr;
    my ($port, $iaddr) = sockaddr_in($sockaddr);
    my $addr = inet_ntoa($iaddr);
    return ($addr, $port);
}

sub Nonblock :Export(:DEFAULT) {
    croak q{usage: Nonblock FILEHANDLE} if @_ != 1;
    my ($fh) = @_;
    croak q{Nonblock: $_[0] isn't FILEHANDLE}
        if ref $fh ne 'GLOB' || !*{$fh}{IO};
    my $old_flags = fcntl($fh, $F_GETFL, 0)         or croak "fcntl: GETFL: $!";
    fcntl($fh, $F_SETFL, $old_flags | $O_NONBLOCK)  or croak "fcntl: SETFL: $!";
    return;
}

sub Accept :Export(:DEFAULT) {
    croak q{usage: Accept SERVER, CB [, METHOD]}  if !(2 <= @_ && @_ <= 3);
    my ($server, $cb, $method) = @_;
    croak q{Accept: $_[0] isn't open FILEHANDLE}    if !openhandle($server);
  ACCEPT: {
    while (my $paddr = accept(my $client, $server)) {
        Nonblock($client);  # XXX   ݣ  ? :-(
        #   ,   IP  PORT  
        #  INET-,   UNIX.
        my @cb_params = ($client);
        if (sockaddr_family($paddr) == $AF_INET) {
            my ($port, $iaddr) = sockaddr_in($paddr);
            my $ip = inet_ntoa($iaddr);
            push @cb_params, $ip, $port;
        }
        run_callback($cb,  $method || 'ACCEPT',  @cb_params);
    }
    redo ACCEPT         if $! == $EINTR;
  }
    croak "accept: $!"  if $! != $EAGAIN;
    return;
}

sub Sysread :Export(:DEFAULT) {
    croak q{usage: Sysread FILEHANDLE, BUFFER}  if @_ != 2;
    \my ($fh, $buf) = \(@_);
    croak q{Sysread: FILEHANDLE isn't open}     if !openhandle($fh);
    croak q{Sysread: BUFFER undefined}          if !defined $buf;
  SYSREAD: {
    my $n;
    while ($n = sysread $fh, my $read_buf, $BLOCK_SIZE) {
        $buf .= $read_buf;
    }
    return 1            if defined $n;  # EOF
    redo SYSREAD        if $! == $EINTR;
  }
    croak "sysread: $!" if $! != $EAGAIN;
    return undef;                       # no EOF
}

sub Syswrite :Export(:DEFAULT) {
    croak  q{usage: Syswrite FILEHANDLE, BUFFER [, POSITION]}
        if !( 2 <= @_ && @_ <= 3 );
    my $has_pos = @_ == 3;
    push @_, 0 unless $has_pos;         # allow alias $pos
    \my ($fh, $buf, $pos) = \(@_);      # alias $fh only to fit in single line
    croak  q{Syswrite: FILEHANDLE isn't open}               if !openhandle($fh);
    croak  q{Syswrite: BUFFER undefined}                    if !defined $buf;
    croak  q{Syswrite: POSITION undefined}                  if !defined $pos;
    croak qq{Syswrite: POSITION $pos < 0}                   if $pos < 0;
    my $len = length $buf;
    croak qq{Syswrite: POSITION $pos >= BUFFER length $len} if $pos >= $len;
  SYSWRITE: {
    my $n;                              # just to catch syswrite()==0
    while ($n = syswrite $fh, $buf, $BLOCK_SIZE, $pos) {
        if ($has_pos) {
            $pos += $n;
        }
        else {
            substr($buf, 0, $n, '');
            $len -= $n;
        }
        return 1 if $pos == $len;       # BUFFER completelly written
        croak qq{Internal error: POSITION $pos > BUFFER length $len}
            if $pos > $len;
    }
    croak 'Internal error: syswrite()==0'   if defined $n;  # NEVER HERE?
    redo SYSWRITE                           if $! == $EINTR;
  }
    croak "syswrite: $!"                    if $! != $EAGAIN;
    return undef;                       # BUFFER partially written
}

sub Select :Export(:DEFAULT) {
    croak  q{usage: Select \%read_fh, \%write_fh [, $timeout]}
        if !( 2 <= @_ && @_ <= 3 );
    my ($r_fh, $w_fh, $timeout) = @_;
    my (%r_fd, %w_fd);                  # fileno for %$r_fh and %$w_fh
    my ($r, $w, $e) = (q{}) x 3;        # input&output bitmask for select()
    for (keys %$r_fh) {
        $r_fd{$_} = $r_fh->{$_} ? fileno($r_fh->{$_}) : undef;
        croak "\$_[0]->{$_} is not open FILEHANDLE" if !defined $r_fd{$_};
        vec($r, $r_fd{$_}, 1) = 1;
    }
    for (keys %$w_fh) {
        $w_fd{$_} = $w_fh->{$_} ? fileno($w_fh->{$_}) : undef;
        croak "\$_[1]->{$_} is not open FILEHANDLE" if !defined $w_fd{$_};
        vec($w, $w_fd{$_}, 1) = 1;
    }
    $e  = $r | $w;
    my ($rc, @r, @w);
    ($rc, $timeout) = select($r, $w, $e, $timeout);
    if ($rc < 0) {
        croak "select: $!" if $! != $EINTR;
    }
    elsif ($rc > 0) {
        $r |= $e;
        $w |= $e;
        @r = grep { vec($r, $r_fd{$_}, 1) } keys %r_fd;
        @w = grep { vec($w, $w_fd{$_}, 1) } keys %w_fd;
    }
    return (\@r, \@w, $timeout);
}


1; # Magic true value required at end of module
__END__

=head1 NAME

POWER::Utils::IO - Make non-blocking IO ease.


=head1 VERSION

This document describes POWER::Utils::IO version 0.6.5


=head1 SYNOPSIS

 use POWER::Utils::IO;

 $socket = tcp_client('google.com', 80);
 
 $socket = udp_client('ns.myisp.com', 53);
 $socket = udp_client();

 $socket = unix_client('/tmp/mysock');

 $socket = tcp_server('ANY', 1234);
 $socket = tcp_server('127.0.0.1', 1234);
 $socket = tcp_server('my.host.com', 1234);

 $socket = udp_server('ANY', 53);

 $socket = unix_server('/tmp/mysock');

 $srv = start_server('127.0.0.1', 1234, \&cb_accept, @params);

 keep_alive($socket, 60, 6, 10);

 my ($ip, $port) = get_peer($socket);

 Nonblock($socket);

 Accept($server, $object, 'MyACCEPT');
 Accept($server, 'MyClass');
 Accept($server, sub {
     my ($client_sock, $ip, $port) = @_;
     ...
 });

 $is_eof = Sysread($fh, $buf);

 $is_done = Syswrite($fh, $buf);
 $is_done = Syswrite($fh, $buf, $pos);

 my %buf  = ( in  => q{}, out => q{some info}, err => q{some warn} );
 my %r_fh = ( in  => \*STDIN );
 my %w_fh = ( out => \*STDOUT, err => \*STDERR );
 my ($r, $w, $timeleft) = Select(\%r_fh, \%w_fh, $timeout);
 for (@$r) {
    my $is_eof = Sysread($r_fh{$_}, $buf{$_});
    delete $r_fh{$_} if $is_eof;
 }
 for (@$w) {
    my $done = eval { Syswrite($w_fh{$_}, $buf{$_}) };
    die "[$_]: $@"   if $@ && $@ !~ /Broken pipe/;
    delete $w_fh{$_} if $@ || $done;
 }


=head1 DESCRIPTION

Make nonblocking I/O ease by providing functions which handle all specific
for nonblocking I/O issues:

 - accept() , sysread() , syswrite() in loop until EAGAIN happens
 - restart accept() , sysread() , syswrite() on EINTR
 - for accept()'ed sockets:
    - make them nonblocking
    - unpack ip/port
 - make buffer manipulation ease:
    - sysread() will append to end of buf and return EOF flag
    - syswrite() will cut written bytes from beginning of buffer
      OR update current position in buffer (like seek() for files)

Also provide non-universal socket creation functions for most usual cases
and select() wrapper for ease multiplexing.


=head1 EXPORTS

start_server() get_peer()

=over

=item :DEFAULT

Nonblock() Accept() Sysread() Syswrite() Select()
tcp_client() udp_client() unix_client()
tcp_server() udp_server() unix_server()
keep_alive()


=back

=head1 INTERFACE

=over

=item tcp_client( HOST, PORT )

Create nonblocking [ AF_INET, SOCK_STREAM, TCP ] socket and
initiate connect() to HOST:PORT.

HOST can be anything inet_aton() understand: hostname or IP address.

Realization is (no croak()ing here for ease read):

 socket(my $sock, AF_INET, SOCK_STREAM, getprotobyname('tcp'));
 Nonblock($sock);
 connect($sock, sockaddr_in($PORT, inet_aton($HOST)));

Return: created socket.


=item udp_client( HOST, PORT )

=item udp_client( )

Create nonblocking [ AF_INET, SOCK_DGRAM, UDP ] socket.
If given HOST & PORT also connect() this socket to HOST:PORT.

HOST can be anything inet_aton() understand: hostname or IP address.

Realization is (no croak()ing here for ease read):

 socket(my $sock, AF_INET, SOCK_DGRAM, getprotobyname('udp'));
 Nonblock($sock);
 connect($sock, sockaddr_in($PORT, inet_aton($HOST))); #optional

Return: created socket.


=item unix_client( PATH )

Create nonblocking [ AF_UNIX, SOCK_STREAM ] socket and
initiate connect() to PATH.

Realization is (no croak()ing here for ease read):

 socket(my $sock, AF_UNIX, SOCK_STREAM, 0);
 Nonblock($sock);
 connect($sock, sockaddr_un($PATH));

Return: created socket.


=item tcp_server( HOST, PORT )

Create nonblocking [ AF_INET, SOCK_STREAM, TCP, SO_REUSEADDR ] listening socket.

HOST can be one of strings (in caps!): 'ANY', 'BROADCAST', 'LOOPBACK'
or anything inet_aton() understand: hostname or IP address.

Realization is (no croak()ing here for ease read):

 my $in_addr = $HOST eq 'ANY'        ? INADDR_ANY
             : $HOST eq 'BROADCAST'  ? INADDR_BROADCAST
             : $HOST eq 'LOOPBACK'   ? INADDR_LOOPBACK
             :                         inet_aton($HOST);
 socket(my $sock, AF_INET, SOCK_STREAM, getprotobyname('tcp');
 setsockopt($sock, SOL_SOCKET, SO_REUSEADDR, pack('L!',1));
 bind($sock, sockaddr_in($PORT, $in_addr));
 listen($sock, SOMAXCONN);
 Nonblock($sock);

Return: created socket.


=item udp_server( HOST, PORT )

Create nonblocking [ AF_INET, SOCK_DGRAM, UDP, SO_REUSEADDR ] socket and
bind it to HOST:PORT.

HOST can be one of strings (in caps!): 'ANY', 'BROADCAST', 'LOOPBACK'
or anything inet_aton() understand: hostname or IP address.

Realization is (no croak()ing here for ease read):

 my $in_addr = $HOST eq 'ANY'        ? INADDR_ANY
             : $HOST eq 'BROADCAST'  ? INADDR_BROADCAST
             : $HOST eq 'LOOPBACK'   ? INADDR_LOOPBACK
             :                         inet_aton($HOST);
 socket(my $sock, AF_INET, SOCK_DGRAM, getprotobyname('udp');
 setsockopt($sock, SOL_SOCKET, SO_REUSEADDR, pack('L!',1));
 bind($sock, sockaddr_in($PORT, $in_addr));
 Nonblock($sock);

Return: created socket.


=item unix_server( PATH )

Create nonblocking [ AF_UNIX, SOCK_STREAM ] listening socket with permissions
0666. B<WARNING>: if PATH exists it will be removed first.

Realization is (no croak()ing here for ease read):

 socket(my $sock, AF_UNIX, SOCK_STREAM, 0);
 unlink($PATH);
 my $umask = umask 0;
 bind($sock, sockaddr_un($PATH));
 umask $umask;
 listen($sock, SOMAXCONN);
 Nonblock($sock);

Return: created socket.


=item start_server( HOST, PORT, CB [, CB_PARAMS] )

Require EV! Start tcp listening socket in background using EV::READ.

Tcp socket will be created using tcp_server(), so you can use any HOST
value supported by tcp_server() like 'ANY'.

If PORT = 0, then will use random port provided by OS.

On accepting incoming connection will make accepted socket non-blocked
and then call CB() with client socket as first param and optional array
CB_PARAMS as other params.

Usage example:

 my $srv = start_server($host, $port, \&cb_srv, @cb_params);
 sub cb_srv {
    my ($sock, @cb_params) = @_;
    ...
 }

Return: hash with fields:

 {sock}     listening server socket
 {port}     server port
 {w}        EV::READ watcher

You have to keep returned hash, because when you drop it EV::READ watcher
will be destroyed, together with listening socket.


=item keep_alive( SOCKET [, IDLE, COUNT, INTERVAL] )

Activate keep-alive feature on this SOCKET. Default values are:

 IDLE       60
 COUNT      6
 INTERVAL   10

Implementation is:

 setsockopt $fh, SOL_SOCKET,  SO_KEEPALIVE,  pack 'l', 1
 setsockopt $fh, IPPROTO_TCP, TCP_KEEPIDLE,  pack 'l', $idle
 setsockopt $fh, IPPROTO_TCP, TCP_KEEPCNT,   pack 'l', $cnt
 setsockopt $fh, IPPROTO_TCP, TCP_KEEPINTVL, pack 'l', $intvl

Return: nothing.

=item get_peer( SOCKET )

Return socket's peer address as ($ip, $port) or ('',0) if socket not connected.

=item Nonblock( FILEHANDLE )

Make FILEHANDLE non-blocking.

Return: nothing.


=item Accept( SERVER, CB [, METHOD] )

accept() all waiting connections (SERVER should be O_NONBLOCK).

If no METHOD given and CB isn't CODE then use method 'ACCEPT'.

For each accepted client socket: make it O_NONBLOCK
and execute callback CB->METHOD(params) using POWER::Utils/run_callback().

CB Params are:

 (CLIENT, $ip, $port)      # for AF_INET
 (CLIENT)                  # for non AF_INET (i.e. AF_UNIX)

Return: nothing.


=item Sysread( FILEHANDLE, $buf )

sysread() until EOF or EAGAIN/other error happens.
Readed data will be added to end of $buf.

Return: true when reach EOF, false otherwise.


=item Syswrite( FILEHANDLE, $buf )

=item Syswrite( FILEHANDLE, $buf, $pos )

syswrite() until $buf will be written or EAGAIN/other error happens.

If $pos given, then will syswrite() from $pos position in $buf and
number of written bytes will be added to $pos.

If $pos isn't given, then will syswrite() from beginning of $buf and
written bytes will be removed from $buf.

Return: true if $buf completelly written, false otherwise.


=item Select( \%read_fh, \%write_fh [, $timeout] )

This is wrapper for select() which will do all bit-masking work.
There two differences between select() and Select() in behaviour:

 1) Select() will throw exception on errors (if fileno()
    can't be found or select() return non-EINTR error).
 2) All filehandles will be checked for errors in addition
    to requested read/write possibility.

%read_fh and %write_fh should contain any keys and filehandles as values
for these keys. Also these hashes can be empty. RBITS and WBITS masks
will be calculated for filehandles in these hashes; EBITS will be calculated
as RBITS|WBITS.

$timeout can be anything accepted as timeout by select(). If unset,
undef() will be used.

Return: 3-element list: (\@read_keys, \@write_keys, $timeleft).
@read_keys will contain keys from %read_fh for filehandles which
can be read or which has error; @write_keys is same for %write_fh -
can be written or has error. $timeleft is second value returned
by select() - how many time left until $timeout in case some I/O
events arise before $timeout expires or select() was interrupted
by signal.

=over

NOTE: select() isn't very effective for multiplexing, so if you need
to multiplex a lot of filehandles better look at L<POWER::Event::IO>,
which use epoll() and callbacks for effective multiplexing.

=back


=back

=head1 DIAGNOSTICS

=over

=item C<< usage: tcp_client HOST, PORT >>

=item C<< tcp_client: $_[0] undefined >>

=item C<< tcp_client: $_[1] undefined >>

tcp_client() need two params... both, at least, defined.


=item C<< can't resolve ... >>

This error can happens in tcp_client() and tcp_server() . It mean what
inet_aton() unable to resolve IP address from host name.


=item C<< socket: $! >>

This error can happens in tcp_client() , tcp_server() , unix_client() and
unix_server() .
See L<socket(2)>.


=item C<< connect: $! >>

This error can happens in tcp_client() and unix_client() .
See L<connect(2)> for details.


=item C<< usage: unix_client PATH >>

=item C<< unix_client: $_[0] undefined >>

unix_client() require one param and it should be defined.


=item C<< usage: tcp_server HOST, PORT >>

=item C<< tcp_server: $_[0] undefined >>

=item C<< tcp_server: $_[1] undefined >>

tcp_server() need two params... both, at least, defined.


=item C<< setsockopt: $! >>

Error happens in tcp_server() .
See L<setsockopt(2)> for details.


=item C<< bind: $! >>

Error happens in tcp_server() or unix_server() .
See L<bind(2)> for details.


=item C<< listen: $! >>

Error happens in tcp_server() or unix_server() .
See L<listen(2)> for details.


=item C<< usage: unix_server PATH >>

=item C<< unix_server: $_[0] undefined >>

unix_server() require one param and it should be defined.


=item C<< usage: Nonblock FILEHANDLE >>

=item C<< Nonblock: $_[0] isn't FILEHANDLE >>

Nonblock() require single param and it must be FILEHANDLE.


=item C<< fcntl: GETFL: $! >>

=item C<< fcntl: SETFL: $! >>

May happens in Nonblock().
I don't sure is fcntl's GETFL/SETFL can return any error, but if they do -
you get it. No idea why. See L<fcntl(2)> for details.


=item C<< usage: Accept SERVER, CB [, METHOD] >>

=item C<< Accept: $_[0] isn't open FILEHANDLE >>

Accept() require two or three params and first must be open FILEHANDLE
(hmm, it must be listening socket of SOCK_STREAM type,
but maybe I wanna too much from you). :-)


=item C<< accept: $! >>

Low-level accept() called inside Accept() dislike something...
probably you out of some resources. See L<accept(2)> for details.


=item C<< usage: Sysread FILEHANDLE, BUFFER >>

=item C<< Sysread: FILEHANDLE isn't open >>

Sysread require exactly two params. First must be open FILEHANDLE.


=item C<< Sysread: BUFFER undefined >>

This is sort of 'use strict' protection. Please give defined buffer to
Sysread() even if it should be empty.


=item C<< sysread: $! >>

May happens in Sysread(). See L<read(2)> for details.


=item C<< usage: Syswrite FILEHANDLE, BUFFER [, POSITION] >>

=item C<< Syswrite: FILEHANDLE isn't open >>

Syswrite() require two or three params. First must be open FILEHANDLE.


=item C<< Syswrite: BUFFER undefined >>

Again, like in Sysread(), sort of 'use strict' protection.
Anyway, it doesn't has much sense trying to write undefined BUFFER... :)


=item C<< Syswrite: POSITION undefined >>

=item C<< Syswrite: POSITION $pos < 0 >>

=item C<< Syswrite: POSITION $pos >= BUFFER length $len >>

If third param to Syswrite() given, then it must be non-negative numeric
value, and can't be greater than BUFFER size. Strange requirements, yeah?


=item C<< Internal error: POSITION $pos > BUFFER length $len >>

=item C<< Internal error: syswrite()==0 >>

If you see one of these errors - you find a BUG in this module.
I was unable even to emulate them in tests... :(


=item C<< syswrite: $! >>

May happens in Syswrite(). See L<write(2)> for details.


=item C<< $_[0]->{$key} is not open FILEHANDLE >>

Select() unable to find fileno() for this $key in %read_fh.

=item C<< $_[1]->{$key} is not open FILEHANDLE >>

Select() unable to find fileno() for this $key in %write_fh.

=item C<< select: $! >>

Some non-EINTR error was returned by select().


=back


=head1 CONFIGURATION AND ENVIRONMENT

POWER::Utils::IO requires no configuration files or environment variables.


=head1 DEPENDENCIES

 Export::Attrs

 POWER::Utils


=head1 INCOMPATIBILITIES

None reported.


=head1 BUGS AND LIMITATIONS

No bugs have been reported.

Please report any bugs or feature requests to
C<bug-power-utils-io@rt.cpan.org>, or through the web interface at
L<http://rt.cpan.org>.


=head1 AUTHOR

Alex Efros  <powerman@cpan.org>


=head1 LICENCE AND COPYRIGHT

Copyright (c) 2005,2006,2014 Alex Efros <powerman@cpan.org>. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself. See L<perlartistic>.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.
